<?php

require_once( SITEROOT . "class/Versionning.class.php");


class Layout {
  public $header;
  public $body;
  public $title;

  /**
   * \brief Mthode permettant de dfinir le titre du document
   * \param $str chane : le titre du document
  */
  public function setTitle($str) {
    $this->title = $str;
  }
  
  /**
   * \brief Mthode d'ajout de contenu dans l'en-tte du document
   * \param $str chane : chane de caractres  ajouter dans le document
  */
  public function addHeader($str) {
    $this->header .= $str;
  }

  /**
   * \brief Mthode d'ajout de contenu dans le corps du document
   * \param $str chane : chane de caractres  ajouter dans le document
  */
  public function addBody($str) {
    $this->body .= $str;
  }
  /**
   * 
   * @param $templateFile: le template full path name correspond to the controller index.php 
   * @return no return value
   */
  public function setTemplate($template)
  {
  	$this->templateFile=$template;
  }
  
 

}

/**
 * \class HTMLLayout Layout.class.php
 * \brief Classe pour la gnration de mise en page en HTML
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 17.02.2006
 * 
 *
 * Cette classe fournit des mthodes pour la gnration de mise en page en HTML
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

class HTMLLayout extends Layout {

	protected $template = false;
	
	private $errorDisabled;
	
  public function __construct($template = false) {
    if ($template) {
      $this->template = $template;
    } elseif (defined("DEFAULT_HTML_TEMPLATE")) {
      $this->template = DEFAULT_HTML_TEMPLATE;
    }
  }
  
  public function disableError(){
  	$this->errorDisabled = true;
  }

  public function openContainer($displayInline = false) {
      $html = "        <div class=\"container\">\n";
      $html .= "            <div class=\"row\">\n";
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }
    
  public function closeContainer($displayInline = false) {
      $html = "            </div><!-- <div class=\"row\" -->\n";
      $html .= "        </div><!-- <div class=\"container\" -->\n";
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }
  
  public function openSideBar($displayInline = false) {
      $html = "                <div id=\"sidebar\" class=\"col-md-3\" role=\"navigation\">\n";
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }

  public function closeSideBar($displayInline = false) {
      $html = "                </div><!-- <div id=\"sidebar\" -->\n";
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }  
  
  public function openContent($displayInline = false) {
      $html = "                <div id=\"content\" class=\"col-md-9\" role=\"main\">\n";
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }

  public function closeContent($displayInline = false) {
      $html = "                </div><!-- <div class=\"content\" -->\n";
      
      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }  
  
  /**
   * \brief Mthode permettant de construire un menu
   * \param $user objet (optionnel) : objet reprsentant l'utilisateur en cours pour personnalisation du menu
   * \param $displayInline boolen (optionnel) : spcifie si le HTML doit tre affich (true) ou ajout au corps du document (false, par dfaut)
  */
  public function buildMenu(User $user = null, $displayInline = false) {

      $sqlQuery = ObjectInstancierFactory::getObjetInstancier()->get('SQLQuery');

      $userSQL = new UserSQL($sqlQuery);
      $userInfo = $userSQL->getInfo($user->getId());
      $moduleSQL = new ModuleSQL($sqlQuery);

      $modulesInfo = $moduleSQL->getModulesForUser($userInfo);

      $menuHTML = new MenuHTML();
      $html =  $menuHTML->getMenuContent($userInfo,$modulesInfo);

      if ($displayInline) {
          echo $html;
      } else {
          $this->addBody($html);
      }
  }
  
  
  /**
   * \brief Mthode de construction du pied de page du document
   * \param $displayInline boolen (optionnel) : spcifie si le HTML doit tre affich (true) ou ajout au corps du document (false, par dfaut)
  */
  public function buildFooter($displayInline = false) {
  	
    $html = "        <footer class=\"bs-footer\">\n            <div class=\"container\">\n";

	if (defined("WEBMASTER")) {
	  $html .= "                <a href=\"mailto:" . WEBMASTER . "\" class=\"link-white\">Webmaster</a> - \n";
	}

	if (defined("SUPPORT_URL")) {
	  $html .= "                <a href=\"" . SUPPORT_URL . "\" class=\"link-white\">Support</a> - \n";
	}

	$versionning = VersionningFactory::getInstance();
	$versionningInfo = $versionning->getAllInfo();
	
	$html .= "                    Offre SLOW - <a href=\"" . WEBSITE_SSL . "/common/release_notes.php\">\n".
  	$versionningInfo['version-complete'] . "</a>\n";
	global $debut;
  	if ($debut){
  		$html .= " - " . round(1000 * (microtime(true) - $debut)) . " ms\n";
  	}
	
	$html .= "            </div>\n        </footer>\n";

    if ($displayInline) {
      echo $html;
    } else {
      $this->addBody($html);
    }
  }

  /**
   * \brief Mthode de construction de la zone de pagination
   * \param $dataObj DataObject : objet reprsentant les donnes manipules et contenant les informations de pagination
   * \param $displayInline boolen (optionnel) : spcifie si le HTML doit tre affich (true) ou ajout au corps du document (false, par dfaut)
  */
  public function buildPager($dataObj, $displayInline = false) {
        $nb_total_page = $dataObj->get("pageNbr");
        $page_number = $dataObj->get("currentPage");
        $page = array(1,2,3,$page_number  - 1 , $page_number , $page_number +1,$nb_total_page-2,$nb_total_page-1,$nb_total_page );
        $page = array_unique($page);
        sort($page);
	foreach($page as $i => $nb_page){
			if ($nb_page>$nb_total_page || $nb_page<=0){
				unset($page[$i]);
			}
        }
        $last_page = 0;	
	$html = "            <div id=\"display-items\">\n";

	// Nombre de rsultats par page
	$html .= "<h2>Afficher par page</h2>\n";
	$html .= "<ul class=\"pagination pagination-sm\">\n";
	foreach (array(10, 20, 50, 100) as $val) {
	  if ($dataObj->get("displayItems") != $val) {
		$html .= "<li><a href=\"" . Helpers::getURLWithParam(array("count" => $val)) . "\" title=\"Afficher " . $val . " lments par page\">" . $val . "</a></li>\n";
	  } else {
                $html .= "<li class=\"disabled\"><a href=\"#\">" . $val . "</a></li>\n";
	  }
	}

	$html .= "</ul>\n</div>\n";

	// Liste des pages
        $html .= "<div id=\"pages\">\n";
	$html .= "<h2>Page&nbsp;:</h2>\n";
	$args = preg_replace("/&?page=[0-9]+/", "", $_SERVER["QUERY_STRING"]);
	$args = preg_replace("/^&/", "", $args);
	$args = preg_replace("/&/", "&amp;", $args);
	$sep = (strlen($args) > 0) ? "&amp;" : "";

	$html .= "<ul class=\"pagination pagination-sm\">\n";
        if ($page_number > 1) {
	  $args = preg_replace("/&?page=[0-9]+/", "", $_SERVER["QUERY_STRING"]);
	  $args = preg_replace("/^&/", "", $args);
	  $args = preg_replace("/&/", "&amp;", $args);
	  $sep = (strlen($args) > 0) ? "&amp;" : "";

	  $html .= "<li><a href=\"" . Helpers::getURLWithParam(array("page" => ($dataObj->get("currentPage") - 1))) . "\" title=\"Afficher la page prcdente\">&laquo;</a></li>\n";
	} else {
	  $html .= "<li class=\"disabled\"><a href=\"#\">&laquo;</a></li>\n";
	}
        foreach ($page as $i) {
            if ($last_page + 1 != $i) {
              $html .= "<li class=\"disabled\"><a href=\"#\">...</a></li>\n";  
            }
            $last_page = $i;
            if ($page_number == $i) {
                $html .= "<li class=\"active\"><a href=\"#\">" . $i . "</a></li>\n";
            } else {
                $html .= "<li><a href=\"" . Helpers::getURLWithParam(array("page" => $i)) . "\" title=\"Afficher la page " . $i . "\">" . $i . "</a></li>\n";
            }
        }
        if ($page_number < $nb_total_page) {
	  $args = preg_replace("/&?page=[0-9]+/", "", $_SERVER["QUERY_STRING"]);
	  $args = preg_replace("/^&/", "", $args);
	  $args = preg_replace("/&/", "&amp;", $args);
	  $sep = (strlen($args) > 0) ? "&amp;" : "";

	  $html .= "<li><a href=\"" . Helpers::getURLWithParam(array("page" => ($dataObj->get("currentPage") + 1))) . "\" title=\"Afficher la page suivante\">&raquo;</a></li>\n";
	} else {
	  $html .= "<li class=\"disabled\"><a href=\"#\">&raquo;</a></li>\n";
	}
	$html .= "</ul>\n</div>\n";


    if ($displayInline) {
      echo $html;
    } else {
      $this->addBody($html);
    }
  }


  /**
   * \brief Mthode de construction d'un champ de formulaire de type select
   * \param $name chane : Nom du champ de formulaire
   * \param $data tableau : tableau ayant pour clef les valeurs et pour valeur les noms des entres du select
   * \param $selectedValue mixed : valeur actuelle du champ pour prselection
   * \param $extraAttributes chane : chane de caractres contenant des attribut HTML  ajouter au select
   * \return Le code HTML du champ select
   * @deprecated 
  */
  public function getHTMLSelect($name, $data, $selectedValue, $extraAttributes = "",$onChange=null) {
  	if ($onChange==null) {
    	$html = "<select class=\"form-control\" name=\"" . $name . "\"" . $extraAttributes . ">\n";
  	} else { 
    	$html = '<select id="'.$name.'" class="form-control" name="' . $name . "\"" . $extraAttributes . ' onchange="'.$onChange.'">\n';
  	}
    $html .= " <option value=\"\">Choisissez</option>\n";

    foreach ($data as $key => $val) {
      $html .= " <option value=\"" . $key . "\"";

      $html .= (strcmp($key, $selectedValue) == 0) ? " selected=\"selected\"" : "";
      
      $html .= ">" . $val . "</option>\n";
    }

    $html .= "</select>\n";

    return $html;
  }
  

  /**
   * \brief Mthode de construction d'un champ de formulaire de type checkbox
   * \param $name chane : Nom du champ de formulaire
   * \param $value mixed : Valeur du champ, si valu  "true" la checkbox sera prcoche
   * \param $extraAttributes chane : Attributs supplmentaires du champ HTML
   * \return Le code HTML du champ checkbox
  */
  public function getHTMLCheckbox($name, $value, $extraAttributes = "") {
    $html = "<input type=\"checkbox\" name=\"" . $name . "\" value=\"on\"";

    if ($value == "on") {
      $html .= " checked=\"checked\"";
    }

    $html .= $extraAttributes . " />";

    return $html;
  }

  /**
   * \brief Mthode de construction d'une ligne de tableau de liste d'attribut
   * \param $name chane : Nom de l'attribut
   * \param $value mixed : Valeur de l'attribut
   * \return Le code HTML de la ligne
  */
  public function getHTMLArrayline($name, $value) {
	$html = " <tr>\n";
	$html .= "  <th class=\"td-register th-row\" scope=\"row\">" . $name . "&nbsp;:</th>\n";
	$html .= "  <td class=\"td-input\">" . $value . "</td>\n";
	$html .= " </tr>\n";

    return $html;
  }


  /**
   * \brief Mthode d'inclusion du message d'erreur stock en session
  */
    public function includeErrors() {
        if ($this->errorDisabled){
            return;
        }

        ob_start();
        if (isset($_SESSION["error"])) { 
            $this->afficheErrors();
        }
        $html = ob_get_contents();
        ob_end_clean();

        //$this->addBody($html);
        // Gros hack moisi  cause d'IE qui bug  l'affichage
        // il faut "injecter" la zone d'erreur  l'intrieur de la zone "content"
        //FIXME (EP), ce n'est pas un "bug" d'IE, la CSS ne dfini la errorbox qu'a l'interieur du content
        //FIXME c'est cette classe qui n'est pas trs bien concu ...
        $this->body = str_replace("role=\"main\">", "role=\"main\">\n" . $html, $this->body);		
    }
  
    public function afficheErrors(){ 
        if (! isset($_SESSION["error"]) || ! $_SESSION["error"] ) {
                return;
        }
        ?>
        <div class="alert alert-warning">
            <strong><?php echo $_SESSION["error"]; ?></strong>
        </div>

        <?php
            unset($_SESSION["error"]); 
    }


    public function displayTemplate($layout,$templateFile)
    {
            $this->includeErrors();
            require_once(HTML_TEMPLATE_PATH . "/" . "new.generic.tpl.php");
    }


    public function addCSS($css){
            $this->addHeader("<link rel='stylesheet' type='text/css' href='$css' />");
    }

    public function addJavascript($javascript){
            $this->addHeader("<script src='$javascript' type='text/javascript'></script>");
    }
  	
	
  /**
   * \brief Mthode gnrant l'affichage du document
  */
 
    public function display() {
        $this->includeErrors();

        if ($this->template) {
			//Note EP 09/09/2015 : avant il y avait require_once, ce qui pour un template est ... spcial.
			//Du coup, ca passe pas les tests unitaire... je mets include, mais je sais pas ce que ca va donner...
            include(HTML_TEMPLATE_PATH . "/" . $this->template);
        } else {
            echo "<?xml version=\"1.0\" encoding=\"iso-8859-15\"?>\n";
            echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n";
            echo "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"fr\">\n";
            echo "    <head>\n";
            echo "        <title>" . $this->title . "</title>\n";
            echo $this->header . "\n";
            echo "    </head>\n";
            echo "    <body>\n";
            echo "        <div class=\"container\">\n";
            echo "            <div class=\"row\">\n";
            echo $this->body . "\n";
            echo "            </div>\n";
            echo "        </div>\n";
            echo "    </body>\n";
        }
    }
}

/**
 * \class CSVLayout Layout.class.php
 * \brief Classe pour la gnration de fichier CSV
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 17.02.2006
 * 
 *
 * Cette classe fournit des mthodes pour la gnration de fichiers CSV
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

class CSVLayout extends Layout {

  /**
   * \brief Mthode d'ajout d'une ligne dans le fichier CSV
   * \param $str chane : chane de caractres  ajouter dans le document ou tableau de champs qui seront ajouts spars par des points virgules
  */
  public function addLine($str) {
	if (is_array($str)) {
	  $line = implode(";", $str);
	} else {
	  $line = $str;
	}

	$line .= "\r\n";

	$this->addBody($line);
  }

  /**
   * \brief Mthode gnrant l'affichage du document
   */
  public function display() {
	$content_type = "text/csv;charset=iso-8859-1";
	if (! empty($this->header)) {
	  $content_type .= ";header=present";
	}

	if (! Helpers::sendFileToBrowser(null, "transactions.csv", $content_type)) {
	  return false;
	}

	if (! empty($this->header)) {
	  echo $this->header . "\r\n";
	}

	echo $this->body;
  }
}

?>
