<?php
require_once(__DIR__."/../../../init/init.php");

$me = new User();

if (! $me->authenticate()) {
  $_SESSION["error"] = "Echec de l'authentification";
  header("Location: " . WEBSITE);
  exit();
}

if (! $me->isAdmin()) {
  $_SESSION["error"] = "Accs refus";
  header("Location: " . WEBSITE_SSL);
  exit();
}

$fauthority = Helpers::getVarFromGet("authority");
$frole =  Helpers::getVarFromGet("role");
$fname = Helpers::getVarFromGet("name");
$fgroup = Helpers::getVarFromGet("group");
$api = Helpers::getVarFromGet("api");


$myAuthority = new Authority($me->get("authority_id"));

$filter = array();
// Construction chane de filtrage
if ($me->isSuper()) { // Le super utilisateur voit toutes les collectivits et tous les groupes
	if (isset($fauthority) && is_numeric($fauthority)) {
		$filter[] .= "users.authority_id=" . addslashes($fauthority);
	}

	if (isset($fgroup) && is_numeric($fgroup)) {
		$filter[] .= "authorities.authority_group_id=" . addslashes($fgroup);
	}
} elseif ($me->isGroupAdmin()) {
  // Un admin de groupe ne voit forcment que les utilisateurs des collectivit appartenant  son groupe
  if (isset($fauthority) && strlen($fauthority) > 0) {
		$auth = new Authority($fauthority);
		if ($auth->isInGroup($me->get("authority_group_id"))) {
			$filter[] .= "users.authority_id='" . addslashes($fauthority) . "'";
		}
	}
	$filter[] .= "authorities.authority_group_id='" . $me->get("authority_group_id") . "'";
} elseif ($me->isAuthorityAdmin()) {
  	//Un admin d'une collectivit ne voit forcment que les utilisateurs de sa collectivit
	$filter[] .= "users.authority_id='" . $me->get("authority_id") . "'";
}

if (isset($frole) && strlen($frole) > 0) {
	$filter[] .= "users.role='" . addslashes($frole) . "'";
}

if (isset($fname) && strlen($fname) > 0) {
	$filter[] .= "users.name ILIKE '%" . addslashes($fname) . "%'";
}

$where = "";
if (count($filter) > 0) {
  $where = "WHERE " . implode($filter, " AND ");
}

// Rcupration de la liste des utilisateurs en fonction du filtre
$users = $me->getUsersList($where);


$statusList = $me->get("statusTypes");
$rolesList = $me->get("roleTypes");


if ($api){
	$jsonOutput->retrictAndDisplay($users,array('id','name','givenname','email','role','status','authority_id','authority_name'));
	exit;
}

if ($me->isAuthorityAdmin()) {
	$title = "Gestion des utilisateurs de la collectivit &nbsp;" . get_hecho($myAuthority->get("name")) . "&nbsp;";
} elseif ($me->isGroupAdmin()) {
	$myGroup = new Group($me->get("authority_group_id"));
	$title = "Gestion des utilisateurs du groupe &nbsp;" . get_hecho($myGroup->get("name")) . "&nbsp;";
} else {
	$title = "Gestion des utilisateurs";
}

$authority_id_list = array();

if ($me->isGroupAdminOrSuper()) {
	if ($me->isGroupAdmin()) {
		$cond = " WHERE authorities.authority_group_id=" . $me->get("authority_group_id")." ORDER BY authorities.name ASC";
	} else {
		$cond = " ORDER BY authorities.name ASC";
	}

 	$authority_id_list = Authority::getAuthoritiesIdName($cond) ;
}

/*****************/

$doc = new HTMLLayout();

$doc->setTitle("Tedetis : gestion des utilisateurs");

$doc->openContainer();
$doc->openSideBar();
$doc->buildMenu($me);
$doc->buildPager($me);
$doc->closeSideBar();
$doc->openContent();

ob_start();?>

<script type="text/javascript" src="/javascript/jfu/js/jquery.min.js"></script> 
<script type="text/javascript" src="/javascript/zselect.js"></script>   
<script type="text/javascript" src="/javascript/zselect_s2low.js"></script>   


<h1><?php echo $title; ?></h1>
<div id="actions-area">
	<h2>Actions</h2>
	<a href="admin_user_edit.php" class="btn btn-primary">Ajouter un utilisateur</a>
</div>
<div id="filtering-area">
	<h2>Filtrage</h2>
		<form action="admin_users.php" method="get" class="form-horizontal">
		<div class="form-group">
			<label for="role" class="col-md-3 control-label">Le rle est</label>
			<div class="col-md-3"><?php echo $doc->getHTMLSelect("role", $me->get("roleTypes"), $frole) ?></div>
			<label for="name" class="col-md-3 control-label">Le nom contient</label>
			<div class="col-md-3">
				<input id="name" class="form-control" type="text" name="name" size="20" maxlength="25" value='<?php echo  (strlen($fname) > 0)?get_hecho($fname):"" ?>' />
			</div>
		</div>
		
		<?php if ($me->isGroupAdminOrSuper()) : ?>
			<div class="form-group">
				<label for="authority" class="col-md-3 control-label">Collectivit</label>
				<div class="col-md-3">
					<select class="form-control zselect_authorities" name="authority" id="authority">
    					<option value="">Toutes</option>
						<?php foreach ($authority_id_list as $key => $val) : ?>
      						<option value="<?php hecho($key) ?>"  <?php echo (strcmp($key, $fauthority) == 0) ? " selected='selected'" : ""; ?>>
      							<?php hecho($val)?> 
      						</option>
    					<?php endforeach; ?>
    				</select>
				</div>
			</div>
		<?php endif;?>
	
		<?php if ($me->isSuper()): ?>
			<div class="form-group">
				<label for="group" class="col-md-3 control-label">Groupe</label>
				<div class="col-md-3"><?php echo $doc->getHTMLSelect("group", Group::getGroupsIdName(), $fgroup) ?></div>
			</div>
		<?php endif; ?>
		
		<div class="form-group">
			<button class="btn btn-default col-md-offset-3 col-md-3" type="submit">Filtrer</button>
		</div>
	</form>
</div>

<h2>Liste des utilisateurs</h2>
<div id="user-list">
	<table class="data-table table table-striped" summary="">
	<thead>
	<tr>
		<th id="name">Nom</th>
		<th id="email">Adresse lectronique</th>
		<th id="role">R&ocirc;le</th>
		<th id="status">Etat</th>
		<th id="authority">Collectivit&eacute;</th>
		<th id="actions">Actions</th>
	</tr>
	</thead>
	<tbody>
	
	<?php foreach ($users as $i => $user) : ?>
	<tr>
		<td headers="name">
			<?php hecho($user["name"]) ?> <?php hecho($user["givenname"]) ?>
			<?php if($user['login']) :  ?>
			(<?php hecho($user["login"]) ?>)
			<?php endif;?>
            <?php $nb_days_before_expire = floor((strtotime($user["cert_not_after"]) - time())/86400) ?>
            <?php if($nb_days_before_expire<1): ?>
                <div class="alert alert-danger message-admin">
                    <b>Certificat expir</b>
                </div>
            <?php elseif ($nb_days_before_expire<30): ?>
                <div class="alert alert-warning message-admin">
                    <b>Certificat expire dans <?php echo $nb_days_before_expire ?> jours</b>
                </div>
            <?php endif; ?>
		</td>
		<td headers="email">
			<a href="mailto:<?php hecho($user["email"]) ?>"><?php hecho($user["email"]) ?></a>
		</td>
  		<td headers="role"><?php echo $rolesList[$user["role"]]  ?></td>
  		<td headers="status"><?php echo $statusList[$user["status"]] ?></td>
  		<td headers="authority">
  			<a href="<?php echo WEBSITE_SSL ?>/admin/authorities/admin_authority_edit.php?id=<?php echo $user["authority_id"]  ?>"><?php hecho($user["authority_name"])?></a>
  		</td>
  		<td headers="actions">
  			<a href="<?php echo WEBSITE_SSL ?>/admin/users/admin_user_edit.php?id=<?php echo $user["id"] ?>" class="icon"><img src="<?php echo WEBSITE_SSL ?>/custom/images/erreur.png" alt="image_modif" title="Modifier" /></a>
  		</td>
  	</tr>
	<?php endforeach; ?>
	</tbody>
	</table>
</div>
	
	
<?php 
$html = ob_get_contents();
ob_end_clean();

$doc->addBody($html);

$doc->closeContent();
$doc->closeContainer();

$doc->buildFooter();

$doc->display();

