<?php

require_once(SITEROOT . "/class/DataObject.class.php");
require_once("ActesTransaction.class.php");
require_once(SITEROOT . "/class/Trace.class.php");

class ActesEnvelope extends DataObject {
  protected $objectName = "actes_envelopes";
  
  protected $user_id;
  protected $siren;
  protected $submission_date;
  protected $department;
  protected $district;
  protected $authority_type_code;
  protected $name;
  protected $telephone;
  protected $email;
  protected $file_path;
  protected $file_size;
  protected $return_mail;
  protected $transactions = array();
  protected $rootDir;
  protected $destDir;
  protected $tmpDir;
  protected $dbFields = array( "user_id" => array( "descr" => "Identifiant utilisateur", "type" => "isInt", "mandatory" => true),
						 "siren" => array( "descr" => "Code SIREN de la collectivit", "type" => "isString", "mandatory" => true),
						 "submission_date" => array( "descr" => "Date de dpt de l'enveloppe", "type" => "isDate", "mandatory" => false),
						 "department" => array( "descr" => "Dpartement de la collectivit", "type" => "isString", "mandatory" => true),
						 "district" => array( "descr" => "Arrondissement de la collectivit", "type" => "isString", "mandatory" => true),
						 "authority_type_code" => array( "descr" => "Code nature de la collectivit", "type" => "isString", "mandatory" => true),
 						 "return_mail" => array( "descr" => "Adresses lectroniques pour la rponse", "type" => "isString", "mandatory" => true),
						 "name" => array( "descr" => "Nom de l'interlocuteur", "type" => "isString", "mandatory" => false),
						 "telephone" => array( "descr" => "Tlphone de l'interlocuteur", "type" => "isString", "mandatory" => false),
						 "email" => array( "descr" => "Adresse lectronique de l'interlocuteur", "type" => "isString", "mandatory" => false),
 						 "file_path" => array( "descr" => "Chemin vers l'archive .tar.gz", "type" => "isString", "mandatory" => false),
 						 "file_size" => array( "descr" => "Taille de l'archive .tar.gz", "type" => "isInt", "mandatory" => false),
						 );
  private $envelope_id;
  private $envXmlFile;
  private $envXmlFileSize;
  private $envXmlObj;
  private $fileList;

  /**
   * \brief Constructeur d'une enveloppe
   * \param id integer Numro d'identifiant d'une enveloppe existante avec laquelle initialiser l'objet
   */
  public function __construct($id = false) {
	parent::__construct($id);
  }

  /**
   * \brief Mthode d'obtention de la liste des identifiants des enveloppes
   * \param $cond (optionnel) chane : Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \param $transmitted_only boolen (optionnel) : Ne considre que les enveloppes rellement transmises si true (false par dfaut)
   * \return Tableau des identifiants des enveloppes
  */
  public static function getEnvelopesId($cond = false, $transmitted_only = false) {
	$sql = "SELECT DISTINCT actes_envelopes.id FROM actes_envelopes"
	  . " LEFT JOIN users ON actes_envelopes.user_id=users.id"
	  . " LEFT JOIN actes_transactions ON actes_transactions.envelope_id=actes_envelopes.id"
	  . " LEFT JOIN actes_transactions_workflow ON actes_transactions_workflow.transaction_id=actes_transactions.id";

	if ($transmitted_only) {
	  if ($cond) {
		$cond .= " AND ";
	  } else {
		$cond = "";
	  }

	  $cond .= " actes_transactions_workflow.status_id=3";
	}

	if ($cond) {
	  $sql .= " WHERE " . $cond;
	}

	$db =DatabasePool::getInstance();

	$result = $db->select($sql);

	if (! $result->isError()) {
	  return $result->get_all_rows();
	}

	return false;
  }

  /**
   * \brief Mthode d'obtention du volume transmis depuis une priode donne ou depuis toujours
   * \param $cond (optionnel) chane : Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \param $transmitted_only boolen (optionnel) : Ne considre que les enveloppes rellement transmises si true (false par dfaut)
   * \return Volume en octet correspondant  la demande
  */
  public static function getEnvelopesVolume($cond = false, $transmitted_only = false) {
	$sql = "SELECT SUM(actes_envelopes.file_size) AS sum FROM actes_envelopes actes_envelopes"
	  . " LEFT JOIN users ON actes_envelopes.user_id=users.id"
	  . " LEFT JOIN actes_transactions ON actes_transactions.envelope_id=actes_envelopes.id"
	  . " LEFT JOIN actes_transactions_workflow ON actes_transactions_workflow.transaction_id=actes_transactions.id";

	$filter = array();

	if ($cond) {
	  $filter[] = $cond;
	}

	if ($transmitted_only) {
	  $filter[] = "actes_transactions_workflow.status_id=3";
	} else {
	  $filter[] = "actes_transactions_workflow.status_id=1";
	}

	// Pour viter les doublons
	$filter[] = "actes_transactions.id=(SELECT MIN(actes_transactions.id) FROM actes_transactions WHERE actes_transactions.envelope_id=actes_envelopes.id)";

	$sql .= " WHERE " . implode(" AND ", $filter);

	$db =DatabasePool::getInstance();

	$result = $db->select($sql);

	if (! $result->isError()) {
	  $row = $result->get_next_row();
	  return $row["sum"];
	}

	return false;
  }

  /**
   * \brief Mthode d'obtention de la liste des fichiers inclus dans une enveloppe
   * \param $env_id integer : Identifiant de l'enveloppe concerne
   * \return Tableau de noms de fichier
  */
  public static function getEnvelopesIncludedFiles($env_id) {
	$sql = "SELECT filename FROM actes_included_files WHERE envelope_id=" . $env_id;

	$db =DatabasePool::getInstance();

	$result = $db->select($sql);

	if (! $result->isError()) {
	  return $result->get_all_rows();
	}

	return false;
  }

  /**
   * \brief Mthode d'obtention d'un historique d'envoi des enveloppes au serveur du ministre
   * \param $authority_id integer (optionnel) : Identifiant de la collectivit expditrice des enveloppes
   * \return Tableau des donnes des enveloppes
  */
  public static function getEnvelopesHistory($authority_id = false) {
	$sql = "SELECT DISTINCT ae.id, ae.file_path, atw.date, auth.siren, auth.department, auth.district";
	$sql .= " FROM actes_envelopes ae";
	$sql .= " LEFT JOIN users ON ae.user_id=users.id";
	$sql .= " LEFT JOIN authorities auth ON users.authority_id=auth.id";
	$sql .= " LEFT JOIN actes_transactions at ON at.envelope_id=ae.id";
	$sql .= " LEFT JOIN actes_transactions_workflow atw ON atw.transaction_id=at.id";
	// On veut rcuprer la date o la transaction a t transmise => statut 3
	$sql .= " WHERE atw.status_id=3";

	if ($authority_id) {
	  $sql .= " AND auth.id=" . $authority_id;
	}

	$sql .= " ORDER BY atw.date DESC";

	$db =DatabasePool::getInstance();

	$result = $db->select($sql);

	$env = array();

	if (! $result->isError()) {
	  while ($row = $result->get_next_row()) {
		$env[] = $row;
	  }
	}

	return $env;
  }

  /**
   * \brief Mthode permettant de fixer la valeur d'un attribut
   * \param $name chane : Nom de l'attribut
   * \param $val : valeur de l'attribut
  */
  public function set($name, $val) {
	switch ($name) {
	case "destDir":
	  parent::set("rootDir", ACTES_FILES_UPLOAD_ROOT);
	  break;
	}

	parent::set($name, $val);
  }

  /**
   * \brief Mthode initialisant l'entit avec l'identifiant courant
   * \return true si succs, false sinon
  */
  public function init() {
	if (! parent::init()) {
	  return false;
	}

	$this->initTransactions();

	return true;
  }

  /**
   * \brief Mthode de rcupration des transactions de l'enveloppe courante dans la variable membre $transactions
   */
  public function initTransactions() {
	if (isset($this->id)) {
	  $this->transactions = ActesEnvelope::getTransactionsForEnvelope($this->envelope_id);
	}
  }

  /**
   * \brief Mthode d'obtention de la liste des transactions pour une enveloppe
   * \param $id integer : Identifiant de l'enveloppe pour laquelle rcuprer les transactions
   * \return Tableau d'objet ActesTransaction correspondant  l'enveloppe spcifie
  */
  public static function getTransactionsForEnvelope($id) {
	$transac = array();

	if (! empty($id)) {
	  $sql = "SELECT actes_transactions.id FROM actes_transactions WHERE actes_transactions.envelope_id = " . $id;

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $obj = new ActesTransaction($row["id"]);
		  if ($obj->init()) {
			$transac[] = $obj;
		  }
		}
	  }
	}

	return $transac;
  }

  /**
   * \brief Mthode de remise  zro de la liste des transactions pour l'enveloppe courante
   */
  public function resetTransactions() {
	$this->transactions = array();
  }

  public function addTransaction($trans) {
	$this->transactions[] = $trans;
  }

  /**
   * \brief Mthode de cration du fichier XML de l'enveloppe
   * \return True en cas de succs, false sinon
   */
  public function generateEnvelopeXMLFile($serial) {

  	if (count($this->transactions) <= 0) {
		$this->errorMsg = "Informations manquantes pour gnrer l'enveloppe (pas de transaction)";
		return false;
  	}

	$this->envXmlFile = $this->destDir . "/" . ACTES_APPLI_NAME . "--" . $this->siren . '--' . date('Ymd') . '-' . $serial . '.xml';


	$xml  = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>\n";
    $xml .= "<actes:EnveloppeCLMISILL xmlns:actes=\"http://www.interieur.gouv.fr/ACTES#v1.1-20040216\" ";
    $xml .= "xmlns:insee=\"http://xml.insee.fr/schema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
    $xml .= "xsi:schemaLocation=\"http://www.interieur.gouv.fr/ACTES#v1.1-20040216 actesv1_1.xsd\">\n";

    $xml .= " <actes:Emetteur>\n";
    $xml .= "  <actes:IDCL insee:SIREN=\"" . Helpers::escapeForXML($this->siren) . "\" actes:Departement=\"" . Helpers::escapeForXML($this->department) . "\" actes:Arrondissement=\"" . Helpers::escapeForXML($this->district) . "\" actes:Nature=\"" . Helpers::escapeForXML($this->authority_type_code) . "\"/>\n";
    $xml .= "  <actes:Referent>\n";
    $xml .= "   <actes:Nom>" . Helpers::escapeForXML($this->name) . "</actes:Nom>\n";
    $xml .= "   <actes:Telephone>" . Helpers::escapeForXML($this->telephone) . "</actes:Telephone>\n";
    $xml .= "   <actes:Email>" . Helpers::escapeForXML($this->email) . "</actes:Email>\n";
    $xml .= "  </actes:Referent>\n";
    $xml .= " </actes:Emetteur>\n";

	if (strlen($this->return_mail) > 0) {
	  $xml .= " <actes:AdressesRetour>\n";

	  $emails = explode('|', $this->return_mail);
	  $emails = array_unique($emails);
	  foreach ($emails as $email) {
		$xml .= "  <actes:Email>" . $email . "</actes:Email>\n";
	  }
	  if(defined("ACTES_MAIL_BACKUP")){
	  	$xml .= "  <actes:Email>" . ACTES_MAIL_BACKUP . "</actes:Email>\n";
	  }
	  $xml .= " </actes:AdressesRetour>\n";
	} else {
	  $this->errorMsg = "Informations manquantes pour gnrer l'enveloppe (pas de mail retour)";
	  return false;
	}

    $xml .= " <actes:FormulairesEnvoyes>\n";


	foreach ($this->transactions as $transac) {
		$xml .= "  <actes:Formulaire>\n";
		$xml .= "   <actes:NomFichier>" . Helpers::escapeForXML(basename($transac->get("xmlFileName"))) . "</actes:NomFichier>\n";
		$xml .= "  </actes:Formulaire>\n";
	}


    $xml .= " </actes:FormulairesEnvoyes>\n";
    $xml .= "</actes:EnveloppeCLMISILL>\n";

	return $this->writeEnvFile($xml);
  }

  /**
   * \brief Mthode d'criture du fichier enveloppe
   * \param $xml chane : contenu du fichier  crire
   * \return
   */
  public function writeEnvFile($xml) {
	if (! Helpers::createDirTree(dirname($this->rootDir . '/' . $this->envXmlFile))) {
	  $this->errorMsg = "Erreur systme de fichiers.";
	  return false;
	}

	if (! file_put_contents($this->rootDir . '/' . $this->envXmlFile, $xml)) {
	  $this->errorMsg = "Erreur systme de fichiers.";
	  return false;
	}


	if (! $this->envXmlFileSize = @filesize($this->rootDir . '/' . $this->envXmlFile)) {
	  $this->errorMsg = "Erreur systme.";
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode de gnration de l'archive finale .tar.gz
   * \return True en cas de succs, false sinon
   */
  public function generateArchiveFile() {
	if (isset($this->envXmlFile)) {
	  if (empty($this->file_path)) {
		// Si le chemin n'est pas vide c'est que l'on est en mode import de .tar.gz
		$this->file_path = dirname($this->envXmlFile) . "/" . ACTES_APPLI_TRIGRAMME . '-' . preg_replace("/^(.*)\.([^.]+)$/", "\${1}", basename($this->envXmlFile)) . '.tar.gz';
	  }


	  if (count($this->transactions) > 0) {
		// En cas d'importation on remet dans l'archive tous les fichiers prsents  l'origine
		// (qui se trouvent dans tmpDir)
		if (empty($this->tmpDir)) {
		  $this->buildFileList();
		}

		$filesDir = $this->rootDir . "/" . $this->destDir;



		$filesDir .= (empty($this->tmpDir)) ? "" : "/" . $this->tmpDir;

		if (! chdir($filesDir)) {
		  $this->errorMsg = "Erreur systme de fichiers.";
		  return false;
		} else {
		  $cmd = 'tar cf - ';

		  if (empty($this->tmpDir)) {
			foreach ($this->fileList as $file) {
			  $cmd .= " " . $file["name"];
			}
		  } else {
			// Mode import, on prend tous les fichiers du rpertoire
			$cmd .= " * ";
		  }

		  $cmd .= ' |gzip -9 > ' . $this->rootDir . "/" . $this->file_path;


		  $status = system($cmd, $ret);

		  if ($status === false || $ret != 0) {
			$this->errorMsg = "Erreur de cration de l'archive. Retour " . $ret;
			return false;
		  }

		  Helpers::fixPerms($this->rootDir . "/" . $this->file_path);
		}
	  } else {
		$this->errorMsg = "Pas de message dans l'enveloppe.";
		return false;
	  }

	} else {
	  $this->errorMsg = "Fichier enveloppe introuvable.";
	  return false;
	}

	if (! $this->file_size = filesize($this->rootDir . '/' . $this->file_path)) {
	  $this->errorMsg = "Erreur systme.";
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode de construction de la liste des fichiers contenu dans l'enveloppe
   */
  public function buildFileList() {
	// Fichier XML de l'enveloppe
	$this->fileList[] = array( "name" => basename($this->envXmlFile), "type" => "text/xml", "size" => filesize(ACTES_FILES_UPLOAD_ROOT . "/" .$this->envXmlFile));

	if (count($this->transactions) > 0) {
	  foreach ($this->transactions as $transac) {
		// Fichier XML de l'acte
		$this->fileList[] = array( "name" => basename($transac->get("xmlFileName")), "type" => "text/xml", "size" => filesize(ACTES_FILES_UPLOAD_ROOT . "/" . $transac->get("xmlFileName")));

		// Fichier de l'acte (optionnel)
		if (isset($transac->files["acte"])) {
		  $this->fileList[] = array( "name" => basename($transac->files["acte"]["name"]), "type" => $transac->files["acte"]["mimetype"], "size" => $transac->files["acte"]["size"]);
		}

		// Pices jointes (optionnelles)
		if (isset($transac->files["attachment"])) {
		  foreach ($transac->files["attachment"] as $attachment) {
			$this->fileList[] = array( "name" => basename($attachment["name"]), "type" => $attachment["mimetype"], "size" => $attachment["size"]);
		  }
		}
	  }
	}
  }

  /**
   * \brief Mthode d'importation d'une enveloppe .tar.gz dj constitue
   * \param $name chane : Nom du fichier original de l'archive
   * \param $path chane : Chemin vers le fichier dans le systme de fichier local
   * \return Un tableau des noms des fichiers mtiers XML contenus dans l'enveloppe, false sinon
   */
	public function importArchiveFile($name, $path) {
		if (! isset($this->destDir)) {
			return false;
		}
		$dest = $this->rootDir . '/' . $this->destDir;

		$this->file_path = $this->destDir . "/" . $name;

		// Cration du rpertoire de stockage de l'archive
  		if (! Helpers::createDirTree($dest)) {
			$this->errorMsg = "Erreur systme de fichiers.";
			return false;
  		}
		//Mise en place de l'archive  son emplacement dfinitif
		if (! move_uploaded_file($path, $this->rootDir . '/' . $this->file_path)) {
			$this->errorMsg = "Erreur systme. Abandon.";
			return false;
		}

		// Vrification du format de l'archive
		if (! $this->checkArchiveType($this->rootDir . '/' . $this->file_path)) {
			$this->errorMsg = "Mauvais format de l'archive (.tar.gz requis).";
			return false;
		}

		// Extraction de la taille de l'archive
		if (! $this->file_size = filesize($this->rootDir . '/' . $this->file_path)) {
			$this->errorMsg = "Erreur systme.";
			return false;
		}

		// Vrification taille et antivirus sur l'archive
		if (! $this->checkArchiveConformity($this->rootDir . '/' . $this->file_path)) {
			return false;
		}

		$this->genTempDirectory();

  // Extraction de l'archive dans un rpertoire temporaire
  if (! $this->extractArchive($this->rootDir . '/' . $this->destDir . "/" . $this->tmpDir)) {
	$this->errorMsg = "Erreur dsarchivage archive. Abandon.";
	return false;
  }

  // Chargement du fichier XML enveloppe
  // Le nom du fichier xml enveloppe est le nom de l'archive moins le prfixe
  // et avec l'extension .xml  la place de .tar.gz
  $this->envXmlFile = preg_replace("/^[^\-]+-/", "", $name);
  $this->envXmlFile = $this->destDir . "/" . $this->tmpDir . "/" . preg_replace("/\.tar\.gz$/", ".xml", $this->envXmlFile);

  $xmlFile = $this->rootDir . "/" . $this->envXmlFile;

  if (! file_exists($xmlFile)) {
	$this->errorMsg = "Impossible de trouver le fichier XML enveloppe correspondant  l'archive.";
	return false;
  } else {
	if (! $this->envXmlFileSize = filesize($xmlFile)) {
	  $this->errorMsg = "Erreur systme.";
	  return false;
	}
  }

  if (($xmlFiles = $this->processXMLEnvelope($xmlFile)) === false) {
	return false;
  }

  // criture de l'enveloppe modifie (ajout mail retour TdT)
  if (! $this->writeEnvFile($this->envXmlObj->AsXML())) {
	return false;
  }

  return $xmlFiles;
  }

  /**
   * \brief Mthode de vrification du type de l'archive
   * \param $path chane : Chemin vers le fichier archive
   * \return True en cas de format .tar.gz, false sinon
   *
   */
  public function checkArchiveType($path) {
	if (! empty($path)) {
	  // mime_content_type() ne semble pas fonctionner
	  // On tente btement de lire l'archive et on rcupre le statut de sortie

	  // Test du format gzip
	  $cmd = "gzip -d -c " . $path;

	  exec($cmd, $output, $ret);

	  if ($ret != 0) {
		return false;
	  }

	  // Test du format tar.gz
	  $cmd = "gzip -d -c " . $path . " | tar tf - >/dev/null 2>&1";

	  if ($ret != 0) {
		return false;
	  } else {
		return true;
	  }
	}

	return false;
  }

  /**
   * \brief Mthode de vrification de l'archive (taille et anti-virus)
   * 		La vrification par l'anti-virus se fait aprs le postage de la transaction
   * \param $path chane (optionnel) : chemin vers l'archive  controler (file_path par dfaut)
   * \return True en cas de succs, false si l'archive n'est pas valide
   */
  public function checkArchiveConformity($path = false) {
	return $this->checkArchiveSize($path);
	//&& $this->checkArchiveSanity($path)
  }

  /**
   * \brief Mthode de vrification de la taille de l'archive
   * \return True en cas de conformit, false sinon
   */
  public function checkArchiveSize() {
	if (isset($this->file_path) && ! empty($this->file_path)) {
	  if ($this->file_size > ACTES_ARCHIVE_MAX_SIZE) {
		$this->errorMsg = "La taille de l'archive est trop leve, " . $this->file_size . " octets (maximum " . ACTES_ARCHIVE_MAX_SIZE . " octets autoriss)";
		return false;
	  } else {
		return true;
	  }
	}
		$this->errorMsg =$this->file_path." n'existe pas ou il est vide.";
	return false;
  }
  
  /**
   * \brief Mthode de gnration d'un rpertoire temporaire avec un nom alatoire
  */
  private function genTempDirectory() {
	$this->tmpDir = Helpers::genTempName();
  }

  /**
   * \brief Mthode d'extraction d'une enveloppe archive .tar.gz
   * \param $dest chane : rpertoire de destination de l'extraction
   * \return True en cas de succs, false sinon
   *
   * L'extraction est faite dans le rpertoire tmpDir
   * en dessous du rpertoire de l'archive.
   *
   */
  public function extractArchive($dest) {
	if (! Helpers::createDirTree($dest)) {
	  $this->errorMsg = "Erreur systme de fichiers.";
	  return false;
	}

	$cmd = "gzip -d -c " . $this->rootDir . '/' . $this->file_path . " | tar xf - --directory " . $dest . " >/dev/null 2>&1";

	exec($cmd, $output, $ret);

	if ($ret != 0) {
	  return false;
	} else {
	  return true;
	}
  }

  /**
   * \brief Mthode d'importation de l'enveloppe XML existante
   * \return Tableau de nom de fichier XML des actes en cas de succs, false sinon
   *
   */
  public function processXMLEnvelope($xmlFile) {
	if (($this->envXmlObj = @simplexml_load_file($xmlFile)) === false) {
	  $this->errorMsg = "Erreur lors de l'importation du fichier XML de l'enveloppe.";
	  return false;
	}

	// Extraction des informations du XML pour initialiser l'enveloppe
	$namespaces = $this->envXmlObj->getDocNamespaces();
	// Rcupration des lments dans le namespace "actes"
	$actesItems = $this->envXmlObj->children($namespaces["actes"]);
	$this->name = Helpers::getFromXMLElt($actesItems->Emetteur->Referent->Nom);
	$this->telephone = Helpers::getFromXMLElt($actesItems->Emetteur->Referent->Telephone);
	$this->email = Helpers::getFromXMLElt($actesItems->Emetteur->Referent->Email);

	// Vrification SIREN enveloppe <=> posteur
	$authority_attr = $actesItems->Emetteur->IDCL->attributes($namespaces["insee"]);
	if (strcmp($this->siren, utf8_decode($authority_attr['SIREN'])) != 0) {
	  $this->errorMsg = "Le numro de SIREN contenu dans l'enveloppe ne correspond pas  celui de l'utilisateur authentifi. Abandon.";
	  return false;
	}

	// Vrification type de collectivit enveloppe <=> posteur
	$authority_attr = $actesItems->Emetteur->IDCL->attributes($namespaces["actes"]);
	if ($this->authority_type_code != utf8_decode($authority_attr['Nature'])) {
	  $this->errorMsg = "Le type de collectivit contenu dans l'enveloppe ne correspond pas  celui de l'utilisateur authentifi. Abandon.";
	  return false;
	}

	// Vrification dpartement enveloppe <=> posteur
	$department = Helpers::getFromXMLElt($authority_attr['Departement']);
	if (strcmp($this->department, $department) != 0) {
	  $this->errorMsg = "Le dpartement de la collectivit contenu dans l'enveloppe ne correspond pas  celui de l'utilisateur authentifi. Abandon.";
	  return false;
	}

	// Vrification arrondissement enveloppe <=> posteur
	$district = Helpers::getFromXMLElt($authority_attr['Arrondissement']);
	if (strcmp($this->district, $district) != 0) {
	  $this->errorMsg = "L'arrondissement de la collectivit contenu dans l'enveloppe ne correspond pas  celui de l'utilisateur authentifi. Abandon.";
	  return false;
	}

	// Ajout adresse de retour du TdT
	if (($this->envXmlObj = $this->xmlAddTDTReturnMail()) === false) {
	  return false;
	}

	// Rafraichissement des lments Actes
	$namespaces = $this->envXmlObj->getDocNamespaces();
	$actesItems = $this->envXmlObj->children($namespaces["actes"]);

	// Rcupration adresses de retour
	$return_mails = $actesItems->AdressesRetour;

	$mails = array();
	foreach ($return_mails->Email as $mail) {
	  $mails[] = sprintf("%s", $mail);
	}

	$this->return_mail = implode('|', $mails);

	// Extraction des noms des fichiers XML dcrivant les messages contenus dans l'enveloppe
	$xmlFiles = array();
	$messages = $actesItems->FormulairesEnvoyes;

	foreach ($messages->Formulaire as $message) {
	  $xmlFiles[] = $this->destDir . "/" . $this->tmpDir . "/" . sprintf("%s", $message->NomFichier);
	}

	return $xmlFiles;
  }

  /**
   * \brief Mthode d'ajout de l'adresse mail de retour du TdT dans le fichier XML enveloppe
   * \return Un objet SimpleXMLElement en cas de succs, false sinon
   *
   */
  public function xmlAddTDTReturnMail() {
	// La version PHP 5.1.2 ne contient pas la mthode addChild sur un SimpleXMLElement
	// (apparition en 5.1.4).
	// Il faut donc passer par DOM pour modifier le fichier XML
	if (($domElt = @dom_import_simplexml($this->envXmlObj)) === false) {
	  $this->errorMsg = "Erreur XML.";
	  return false;
	}

	$dom = new DOMDocument('1.0', 'iso-8859-1');
	$domElt = $dom->importNode($domElt, true);
	$dom->appendChild($domElt);

	// TODO : voir comment rcuprer l'URI du namespace depuis le fichier
	$newMail = new DOMElement("actes:Email", ACTES_TDT_MAIL_ADDRESS, "http://www.interieur.gouv.fr/ACTES#v1.1-20040216");

	$mailList = $dom->getElementsByTagName("AdressesRetour");
	  if ($mailList->length <= 0) {
		  $this->errorMsg = "L'enveloppe ne contient pas la balise AdressesRetour";
		return false;
	  }
	  $mailList->item(0)->appendChild($newMail);
	return simplexml_import_dom($dom);
  }

  /**
   * \brief Mthode de vrification antivirus de l'archive
   * \param $path chane (optionnel) : chemin vers l'archive  controler (file_path par dfaut)
   * \return True en cas de succs, false si l'archive est non saine
   */
  public function checkArchiveSanity($path = false) {
	if ($path || (isset($this->file_path) && ! empty($this->file_path))) {
	  if (! $path) {
		$path = $this->rootDir . '/' . $this->file_path;
	  }

		$new_file = ANTIVIRUS_TMP_PATH . basename($path);
		Trace::wrap_exec("cp $path $new_file",$output, $ret);

		if ( $ret != 0 ){
			$t = Trace::getInstance();
			$this->errorMsg="Impossible de copier $path vers $new_file";
			$t->log("Impossible de copier $path vers $new_file",Trace::$TRACE_ERROR);
			return false;
		}

		Trace::wrap_exec("chmod 644 $new_file",$output, $ret);

		Trace::wrap_exec(ANTIVIRUS_COMMAND . " " . $new_file, $output, $ret);
		Trace::wrap_exec("rm $new_file",$output2, $ret2);


	  switch ($ret) {
	  case 0:
		return true;
		break;
	  case 1:
		$this->errorMsg = "L'archive est infecte par un virus. Retour de l'antivirus&nbsp;:<br />\n";
		// Format de ligne : /Nom/de/fichier: Nom virus
		foreach ($output as $line) {
		  if (preg_match('/^\/.*: .* FOUND$/', $line)) {
			$line = explode(":", $line);
			$this->errorMsg .= basename($line[0]) . " : " . $line[1] . "<br />\n";
		  }
		}
		return false;
		break;

	  default:
		  $result = implode("\n",$output);
	  	mail(EMAIL_ADMIN, "Erreur lors du scan antivirus des actes","Message de l'antivirus : $result");
	  	throw new Exception("Clamdscan unexpected error : " . $result);
	  }
	}
	$this->errorMsg="Error: path=".$path." and this->file_path=".$this->file_path;
	return false;
  }

  /**
   * \brief Mthode de suppression des fichiers intermdiaires ayant servi  la contruction de l'archive
   * \return True en cas de succs, false sinon
  */
  public function purgeFiles() {
	// Suppression du fichier XML de l'enveloppe
	if (isset($this->envXmlFile) &&!empty($this->envXmlFile)) {
	  if (! @unlink($this->rootDir . '/' . $this->envXmlFile)) {
		$this->errorMsg .= "Ne peut supprimer un fichier temporaire.";
		return false;
	  }
	}

	// Suppression des fichiers de chaque transaction
	foreach ($this->transactions as $trans) {
	  if (! $trans->purgeFiles()) {
		$this->errorMsg .= $trans->getErrorMsg();
		return false;
	  }
	}

	// Suppression du rpertoire temporaire s'il existe
	if (! empty($this->tmpDir)) {
	  $cmd = "rm -rf " . $this->rootDir . "/" . $this->destDir . "/" . $this->tmpDir;

	  exec($cmd, $output, $ret);

	  if ($ret != 0) {
		return false;
	  }
	}

	return true;
  }

  /**
   * \brief Mthode de contrle de la cloture de toutes les transactions d'une enveloppe puis suppression de l'archive
   * \return True en cas de suppression, false sinon
  */
  public function deleteArchiveFileIfAllClose() {
	$this->initTransactions();

	if (is_array($this->transactions)) {
	  $del = true;

	  reset($this->transactions);
	  while (list($key, $trans) = each($this->transactions) && $del) {
		if (! $trans->isClose()) {
		  $del = false;
		}
	  }

	  if ($del) {
		if ($this->deleteArchiveFile()) {
		  return true;
		} else {
		  return false;
		}
	  }
	}

	return false;
  }

  /**
   * \brief Mthode de suppression du fichier archive .tar.gz
   * \return True en cas de succs, false sinon
  */
  public function deleteArchiveFile() {
	if (isset($this->file_path)) {
	  if (! @unlink(ACTES_FILES_UPLOAD_ROOT . "/" . $this->file_path)) {
		$this->errorMsg .= "Erreur lors de la tentative de suppression du fichier archive.";
		return false;
	  } else {
		// Tentative de suppression du rpertoire contenant
		@rmdir(ACTES_FILES_UPLOAD_ROOT . "/" . dirname($this->file_path));
		return true;
	  }
	}

	return false;
  }



  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode qui renvoie le fichier archive .tar.gz au navigateur
   */
  public function sendFile() {
	$ret_value = false;

	if (isset($this->file_path)) {
	  $archiveFile = ACTES_FILES_UPLOAD_ROOT . "/" . $this->file_path;
	  if (file_exists($archiveFile)) {
		if (! Helpers::sendFileToBrowser($archiveFile, basename($this->file_path), "application/x-gzip")) {
		  $this->errorMsg = "Erreur envoi fichier";
		} else {
		  $ret_value = true;
		}
	  } else {
		$this->errorMsg = "Le fichier archive n'est pas/plus disponible.";
	  }
	}

	return $ret_value;
  }

  /**
   * \brief Mthode d'obtention du numro de srie suivant pour la gnration du nom du fichier XML enveloppe
   * \param $authority Authority (optionnel) : objet reprsentant la collectivit demandeuse du numro
   * \return Un numro de srie
  */
	public function getNextEnvelopeSerial($authority_id) {
		if ( ! $authority_id && ! isset($this->user_id)) {
			return false;
		}
		if (! $authority_id) {
			$user = new User($this->user_id);
			$user->init();
			$authority_id =  $user->get("authority_id");
	  	}


	  if (! $this->db->begin()) {
		$this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
		return false;
	  }

	  // On vrifie si le numro doit tre remis  zro ou pas
	  $sql = "SELECT reset_date, serial FROM actes_envelope_serials WHERE authority_id=$authority_id FOR UPDATE";

	  $result = $this->db->select($sql);

	  if (! $result->isError()) {
		if ($result->num_row() <= 0) {
		  // Pas encore d'entre pour cette collectivit
		  $date = date('Y-m-d');
		  $sql = "INSERT INTO actes_envelope_serials (authority_id, reset_date, serial) VALUES($authority_id, '$date', 2)";

		  if (! $this->db->exec($sql)) {
			$this->errorMsg = "Erreur d'accs base de donnes.";
			$this->db->rollback();
			return false;
		  } else {
			$this->db->commit();
			return 1;
		  }
		} else {
		  $row = $result->get_next_row();
		  $date = substr($row["reset_date"], 0, 10);
		  $today = date('Y-m-d');

		  if (strcmp($today, $date) != 0) {
			// La dernire remise  zro n'est pas d'aujourd'hui => on remet  zro le compteur
			$sql = "UPDATE actes_envelope_serials SET reset_date='" . $today . "', serial=2 WHERE authority_id=$authority_id" ;

			if (! $this->db->exec($sql)) {
			  $this->errorMsg = "Erreur d'accs base de donnes.";
			  $this->db->rollback();
			  return false;
			} else {
			  $this->db->commit();
			  return 1;
			}
		  } else {
			// Mise  jour aujourd'hui => on incrmente le numro de srie

		  // pour le bug 240, modifi par HTan, 15-12-2008
			if ($row['serial']>=10000 || $row['serial']<=0)
			{
				$this->errorMsg = "Serial number: dborder 10000. Vrifier serial est modifi par quelqu'un ou on a commit plus que 9999 tranactions par jour.";
				return false;
			}
			//-----

			$sql = "UPDATE actes_envelope_serials SET serial=serial+1 WHERE authority_id=$authority_id";

			if (! $this->db->exec($sql)) {
			  $this->errorMsg = "Erreur d'accs base de donnes.";
			  $this->db->rollback();
			  return false;
			} else {
			  $this->db->commit();
			  return $row["serial"];
			}
		  }
		}
	  } else {
		$this->errorMsg = "Erreur d'accs base de donnes.";
		return false;
	  }

  }

  /**
   * \brief Mthode d'enregistrement d'une enveloppe dans la base de donnes
   * \param $validate boolen (optionnel) Demande la validation ou non des donnes de l'entit avant enregistrement (true par dfaut)
   * \return true si succs, false sinon
   */
  public function save($validate = true,$bouchon_4_strict_standard = true) {
	$new = false;
	if ($this->isNew()) {
	  $new = true;
	  $this->submission_date = date('Y-m-d H:i:s');
	}

    if (! ($sql = parent::save($validate, true))) {
	  return false;
	}


	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde de la transaction.";
	  $this->db->rollback();
      return false;
    }

	if ($new) {
	  // Ajout du fichier enveloppe dans la liste des fichiers inclus
	  $sql = "INSERT INTO actes_included_files (envelope_id, filename, filetype, filesize) VALUES(" . $this->id . ", '" . pg_escape_string(basename($this->envXmlFile)) . "', 'text/xml', " . $this->envXmlFileSize . ")";

	  if (! $this->db->exec($sql)) {
		$this->errorMsg = "Erreur lors de la journalisation des fichiers contenus dans l'archive.";
		$this->db->rollback();
		return false;
	  }
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

    return true;
  }

  /**
   * \brief Mthode de suppression d'une enveloppe de la base de donnes
   * \param $id integer (optionnel) Numro d'identifiant de l'enveloppe, si non spcifi, entit en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && ! empty($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

	$sql = "DELETE FROM actes_included_files WHERE envelope_id=" . $id;

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression des fichiers relis  l'enveloppe.";
	  $this->db->rollback();
	  return false;
    }

	// On ne supprime pas les transaction tant donn que l'appel  cette mthode se fait
	// uniquement quand l'enregistrment d'une transaction choue

	if (! parent::delete($id)) {
	  $this->db->rollback();
      return false;
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

	return true;
  }

  /**
   * \brief Mthode d'obtention de la liste des enveloppes et tous leurs attributs
   * \param $cond (optionnel) chane : Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \return Tableau des enveloppes
   */
  public function getEnvelopesList($cond = "") {
	if (! $this->pagerInit('DISTINCT actes_envelopes.id, actes_envelopes.user_id, actes_envelopes.siren, ' .
			'actes_envelopes.submission_date, actes_envelopes.department, actes_envelopes.district, ' .
			'actes_envelopes.authority_type_code, actes_envelopes.name, actes_envelopes.telephone, ' .
			'actes_envelopes.email, actes_envelopes.file_path, actes_envelopes.return_mail, ' .
			'actes_envelopes.file_size, users.name, users.givenname ', 'actes_envelopes LEFT JOIN actes_transactions ON ' .
					'actes_envelopes.id=actes_transactions.envelope_id LEFT JOIN users ON actes_envelopes.user_id=users.id', $cond)) {
	  return false;
	}

    return $this->data;
  }
}
